#!/usr/bin/perl -w

#    CCAdmin - Administration script for CGIComment
#    Copyright (C) 2000, 2001, 2004  Markus Triska
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
#
#    Contact me at triska@gmx.at

use strict;
use CGI;

#######################################
# Please change this value here as soon as possible:
my $admin_password = 'admin';

######################################


my %CommentInfo = ();
my $cgi = new CGI;
my $basedir = '../cgicomment/';
my @ShortWeekDays = ('Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun');

my %FullWeekDay = (
	Mon => 'Monday', Tue => 'Tuesday', Wed => 'Wednesday',
	Thu => 'Thursday', Fri => 'Friday', Sat => 'Saturday', Sun => 'Sunday'
);

my %NumMonth = (
	Jan => "01", Feb => "02", Mar => "03", Apr => "04", May => "05",
	Jun => "06", Jul => "07", Aug => "08", Sep => "09", Oct => "10",
	Nov => "11", Dec => "12"
);
my %DateTime = ();

sub filterUserInput
{
	my $string = shift;
	if (!(defined $string)) { return }

	# This code was contributed by b0iler@hotmail.com . Many thanks!
	$string =~ s/([\&;\`'\|\"*\?\~\^\(\)\[\]\{\}\$\n\r])/\\$1/g; #meta characters
	$string =~ s/\0//g; #null byte
	$string =~ s/\.\.//g; #then ..

	return $string;
}


sub FillDateTime
{
	my $ltime = shift;
	if (!$ltime) {
		$ltime = localtime;
	}

	if (substr ($ltime, 8, 1) =~ /\s+/) {
		substr ($ltime, 8, 1) = '0';
	}

	(my $weekday, my $monthstr, my $day, my $Time, my $year) = split (/ /, $ltime);
	$DateTime{'weekday'} = $FullWeekDay{ (substr $weekday, 0, 3)};
	$DateTime{'time'} = $Time;
	$DateTime{'day'} = $day;
	$DateTime{'monthstr'} = $monthstr;
	$DateTime{'month'} = $NumMonth{$monthstr};
	$DateTime{'year'} = $year;
	$DateTime{'hour'} = substr $Time, 0, 2;     # 16:03:22
	$DateTime{'minute'} = substr $Time, 3, 2;
	$DateTime{'second'} = substr $Time, 6, 2;
}

sub FillCommentInfo
{
	my $commentnumber = shift;
	$CommentInfo{'commentnum'} = $commentnumber;

	my $commentpath = $basedir . "comments/$commentnumber";

	$CommentInfo{'text'} = FileToScalar("$commentpath.msg");

	open HDL, "<$commentpath.hdl";
	my $headline = <HDL>;
	chomp $headline;
	close HDL;
	$CommentInfo{'headline'} = $headline;


	open COMMENTINFO, "<$commentpath.inf";
	my $infoline = <COMMENTINFO>;
	chomp $infoline;
	my ($author, $time, $email) = split /\|/, $infoline;
	close COMMENTINFO;
	FillDateTime($time);
	my $authornum = 0;
	if ($author =~ /^REGISTERED/) {
		($authornum = $author) =~ s/REGISTERED//;
		open AUTHOR, "<$basedir". "users/" . "$authornum.syn";
		$author = <AUTHOR>;
		close AUTHOR;
		chomp $author;
	}
	if ($email =~ /seeeml/) {
		open EMAIL, "<$basedir" . "users/" . "$authornum.eml";
		$email = <EMAIL>;
		close EMAIL;
		chomp $email;
	}
	$CommentInfo{'author'} = $author;
	$CommentInfo{'syntax'} = $author;
	$CommentInfo{'email'} = $email;
}

sub FileToScalar
{
	my $file = pop;
	my $scalar = '';
	open FILE, "<$file";
	while (<FILE>) {
		$scalar .= $_;
	}
	close FILE;
	return $scalar;
}


sub ListComments
{
	my $article = shift;
	chomp $article;
	my $articlenum = 0;
	my $articlefound = 0;

	open ARTICLES, "<$basedir" . 'comments/articles.db';
	while (<ARTICLES>) {
		($articlenum, my $file) = split /\|/, $_;
		chomp $file;
		if ($article eq $file) {
			$articlefound = 1;
			last;
		}
	}
	close ARTICLES;
	if (!$articlefound) {
		print "<html><head><title>Error</title></head><body>Article not found</body></html>";
		return;
	}
	open COMMENTS, "<$basedir" . 'comments/comments.db';
	my @comments = <COMMENTS>;
	close COMMENTS;
	print "<html><head><title>Comments for $articlenum</title></head><body><br><br>";
	my $count = 1;
	my $showdeleted = $cgi->param('showdeleted') || 0;
	foreach my $comment (grep /^$articlenum\|/, @comments) {
		my ($articlenum, $commentnum)=split /\|/, $comment;
		chomp $commentnum;
		my $delpath = $basedir . "comments/$commentnum.del";
		if (-e $delpath && !$showdeleted) {
				next;
		}

		FillCommentInfo($commentnum);
		print "<a href=\"/cgi-bin/ccadmin.pl?commentnum=$CommentInfo{'commentnum'}\&action=changecomment\">$count $CommentInfo{'headline'}</a>";
		print " <a href=\"/cgi-bin/ccadmin.pl?commentnum=$CommentInfo{'commentnum'}\&action=delcomment\">";
		if (-e "$basedir" . "comments/$CommentInfo{'commentnum'}.del") {
			print "<font color=\"\#FF0000\">Restore</font>";
		} else {
			print "Delete";
		}
		print "</a><br>";
		$count += 1;
	}
	print "<br><br><br>End of comments</body></html>";
}

sub DelComment
{
	my $commentnum = shift;
	print<<EOF;
<html><head><title>Delete a comment </title></head><body>
<form method="POST" action="/cgi-bin/ccadmin.pl">
<input type="hidden" name="action" value="reallydelete">
<input type="hidden" name="commentnum" value="$commentnum">
This action requires a password:
<input type="password" name="password" value="">
<input type="submit" value="Submit">
</form>
</body>
</html>
EOF

}

sub ReallyDelete
{
	my $commentnum = shift;
	my $password = $cgi->param('password');
	chomp $password;
	if ($password ne $admin_password) {
		print "<html><head><title>Wrong password</title></head><body>Wrong password!</body></html>";
		return;
	}
	my $commentpath = $basedir . "comments/$commentnum";


	my $done = "deleted";
	if (!-e "$commentpath.del") {
		open DELETED, ">$commentpath.del";
		print DELETED "This comment is deleted!";
	} else {
		unlink "$commentpath.del";
		$done = "restored";
	}
	print<<EOF;
<html>
<head><title>Yeah!</title></head><body>
Comment $done.
</body>
</html>
EOF
}

sub ChangeComment
{
	my $commentnum = shift;
	FillCommentInfo($commentnum);
	print<<EOF;
<html><head><title>Change a comment </title></head><body>
<form method="POST" action="/cgi-bin/ccadmin.pl">
<input type="hidden" name="action" value="reallychange">
<input type="hidden" name="commentnum" value="$commentnum">
Headline: <input type="text" name=\"headline" value="$CommentInfo{'headline'}"><br>
 <textarea rows="10" name="text" cols="50" wrap="virtual">$CommentInfo{'text'}
	</textarea>
<br><br>
Password:
<input type="password" name="password" value="">
<input type="submit" value="Submit">
</form>
</body>
</html>
EOF
}

sub ReallyChange
{
	my $commentnum = shift;
	my $password = $cgi->param('password');
	chomp $password;
	if ($password ne $admin_password) {
		print "<html><head><title>Wrong password</title></head><body>Wrong password!</body></html>";
		return;
	}

	my $text = $cgi->param('text');
	my $headline = $cgi->param('headline');


	open MSG, ">$basedir" . "comments/$commentnum.msg";
	print MSG $text;
	close MSG;

	open HDL, ">$basedir" . "comments/$commentnum.hdl";
	print HDL $headline;
	close HDL;
	print "<html><head><title>Done</title></head><body>Entry changed.</body></html>";


}

# Execution entry-point

# Read configuration

if (-e "cgicomment.conf") {
	open CONFIG, "<cgicomment.conf";
	while (<CONFIG>) {
		if (/^\#/) { next; }
		chomp;
		(my $variable, my $value) = split (/=/, $_, 2);
		if ($variable =~ /basedir/i) {
			$value =~ s/(.*)\s+$/$1/;
			if ( substr($value, -1, 1) ne '/') { $value .= '/'; }
			$basedir = $value;
		}
 	}
	close CONFIG;
}


print "Content-type: text/html\n\n";

$CommentInfo{'basedir'} = $basedir;
if ((!$cgi->param('action')) || ($cgi->param('action') eq 'listcomments')) {
	my $article = filterUserInput($cgi->param('article'));
	if (!(defined $article) || $article eq '') {
		print "No article specified.";
	} else {
		ListComments($article);
	}
} elsif ($cgi->param('action') eq 'delcomment') {
	DelComment(filterUserInput($cgi->param('commentnum')));
} elsif ($cgi->param('action') eq 'reallydelete') {
	ReallyDelete(filterUserInput($cgi->param('commentnum')));
} elsif ($cgi->param('action') eq 'changecomment') {
	ChangeComment(filterUserInput($cgi->param('commentnum')));
} elsif ($cgi->param('action') eq 'reallychange') {
	ReallyChange(filterUserInput($cgi->param('commentnum')));
}

