#!/usr/bin/perl -w

#    CGIComment - Add Comments to Arbitrary HTML-Files
#    Copyright (C) 2000, 2001, 2003, 2004 Markus Triska
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
#
#    Contact me via triska@gmx.at


use strict;
use CGI;

# CONFIG SECTION - please do not change these values here, but adjust them via
# cgicomment.conf. This way, you can update to new releases very smoothly.

my $basedir = '../cgicomment/';
my $articledir = '../cginews/sections/';
my $showcommentsent = 1;
my $mailorsite_email = 'mailto:##email##';
my $commentcount = '##number##';
my $commentscount = '##number##';
my $embed_commentcount = '##number##';
my $embed_commentscount = '##number##';
my $totalcomments = '##number##';
my $embed_totalcomments = '##number##';

my %CommentInfo = ();
my @ShortWeekDays = ('Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun');

my %FullWeekDay = (
	Mon => 'Monday', Tue => 'Tuesday', Wed => 'Wednesday',
	Thu => 'Thursday', Fri => 'Friday', Sat => 'Saturday', Sun => 'Sunday'
);

my %NumMonth = (
	Jan => "01", Feb => "02", Mar => "03", Apr => "04", May => "05",
	Jun => "06", Jul => "07", Aug => "08", Sep => "09", Oct => "10",
	Nov => "11", Dec => "12"
);
my %DateTime = ();

my $cgi = new CGI;


# This is a completely unnecessary "encryption" routine. Password files can
# not be retrieved any more when Apache is configured properly due to the
# .htaccess file stored in "/users/".
# Kept for backwards compatibility (sorry!).
sub EncDec
{
	my @args = split //, shift();
	my $retval = '';
	for (my $stringpos = 0; $stringpos <= $#args; $stringpos++) {
		$retval .= chr(ord($args[$stringpos]) ^ 0x12);
	}
	return $retval;
}

sub filterUserInput
{
	my $string = shift;
	if (!(defined $string)) { return; }

	# This code was contributed by b0iler@hotmail.com . Many thanks!
	$string =~ s/([\&;\`'\|\"*\?\~\^\(\)\[\]\{\}\$\n\r])/\\$1/g; #meta characters
	$string =~ s/\0//g; #null byte
	$string =~ s/\.\.//g; #then ..

	return $string;
}


sub FillDateTime
{
	my $ltime = shift;
	if (!$ltime) {
		$ltime = localtime;
	}

	if (substr ($ltime, 8, 1) eq ' ') {
		substr ($ltime, 8, 1) = '0';
	}

	(my $weekday, my $monthstr, my $day, my $Time, my $year) = split (/ /, $ltime);
	$DateTime{'weekday'} = $FullWeekDay{ (substr $weekday, 0, 3)};
	$DateTime{'time'} = $Time;
	$DateTime{'day'} = $day;
	$DateTime{'monthstr'} = $monthstr;
	$DateTime{'month'} = $NumMonth{$monthstr};
	$DateTime{'year'} = $year;
	$DateTime{'hour'} = substr $Time, 0, 2;     # 16:03:22
	$DateTime{'minute'} = substr $Time, 3, 2;
	$DateTime{'second'} = substr $Time, 6, 2;
}

sub GuessMailOrSite()
{
	# hidden feature: "mailorsite" expands to "mailto:##email##" if the user
        # entered an e-mail address, and to "##site##" if the user entered a site.
	# site takes precedence.

	# see what the user entered as her e-mail address. check if it is a URL
	$CommentInfo{'mailorsite'} = SubstRefVars($mailorsite_email);
	my $s = $CommentInfo{'site'};
	if (defined $s && $s ne 'http://' && $s ne '') {
		$CommentInfo{'mailorsite'} = $CommentInfo{'site'};
	}

	# my $m = $CommentInfo{'email'};
	# if ($m !~ /\@/ || $m =~ /\//) {
	#	# it's not a valid mail address - assume URL
	#	$CommentInfo{'mailaddress'} = '';
	#	if ($m !~ /^http\:\/\//) {
	#		$CommentInfo{'site'} = "http://$m";
	#	} else {
	#		$CommentInfo{'site'} = $m;
	#	}
	#	$CommentInfo{'mailorsite'} = $CommentInfo{'site'};
	# } else {
	#	$m =~ s/^mailto://;
	#	$CommentInfo{'site'} = '';
	#	$CommentInfo{'mailaddress'} = $m;
	#	$CommentInfo{'mailorsite'} = SubstRefVars($mailorsite_email);
	# }
}

sub LoadCommentInfo
{
	my $commentnumber = shift;
	$CommentInfo{'commentnum'} = $commentnumber;

	my $commentpath = $basedir . "comments/$commentnumber";

	$CommentInfo{'text'} = FileToScalar("$commentpath.msg");

	open HDL, "<$commentpath.hdl";
	my $headline = <HDL>;
	chomp $headline;
	close HDL;
	$CommentInfo{'headline'} = $headline;


	open COMMENTINFO, "<$commentpath.inf";
	my $infoline = <COMMENTINFO>;
	chomp $infoline;
	my ($author, $time, $email) = split /\|/, $infoline;
	close COMMENTINFO;
	FillDateTime($time);
	my $authornum = 0;
	if ($author =~ /^REGISTERED/) {
		($authornum = $author) =~ s/REGISTERED//;
		my $authorfile = $basedir . "users/$authornum.syn";
		if (-e $authorfile) {
			open AUTHOR, "<$authorfile";
			$author = <AUTHOR>;
			close AUTHOR;
			chomp $author;
		}
	}
	if ($email =~ /seeeml/) {
		my $mailfile = $basedir . "users/$authornum.eml";
		if (-e $mailfile) {
			open EMAIL, "<$mailfile";
			$email = <EMAIL>;
			close EMAIL;
			chomp $email;
		}
	}

	my $sitefile = "$commentpath.sit";
	my $site = '';
	if (-e $sitefile) {
		open SITE, "<$sitefile";
		$site = <SITE>;
		close SITE;
		chomp $site;
	}

	$CommentInfo{'author'} = $author;
	$CommentInfo{'syntax'} = $author;
	$CommentInfo{'email'} = $email;
	$CommentInfo{'site'}  = $site;
	GuessMailOrSite();
}



sub SubstSpecialChars
{
	my $string = shift;

	$string =~ s//\&auml\;/g; $string =~ s//\&Auml\;/g;
	$string =~ s//\&ouml\;/g; $string =~ s//\&Ouml\;/g;
	$string =~ s//\&uuml\;/g; $string =~ s//\&Uuml\;/g;

	$string =~ s/\n/\<br\>/g;
	$string =~ s//\&szlig\;/g;

	# Add more special characters here

	return $string;
}


sub SubstRefVars
{
	my $template = shift;
	while ($template =~ /\#\#.*?\#\#/) {
		(my $refvar = $template) =~ s/.*?\#\#(.*?)\#\#.*/$1/s;
		chomp $refvar;
		my $referredvar = '';
		if (exists $CommentInfo{"$refvar"}) {
			$referredvar = $CommentInfo{"$refvar"};
		} elsif ( exists $DateTime{"$refvar"} ) {
			$referredvar = $DateTime{"$refvar"};
		} else {
			$referredvar = "Warning...could NOT substitute $refvar";
		}
		if ($refvar !~ /^orig/) {
			$referredvar = SubstSpecialChars($referredvar);
		}
		$template =~ s/\#\#.*?\#\#/$referredvar/;
	}
	return $template;
}

sub FileToScalar
{
	my $file = pop;
	my $scalar = '';
	open FILE, "<$file";
	while (<FILE>) {
		$scalar .= $_;
	}
	close FILE;
	return $scalar;
}

sub FetchCommentInfo
{
	# differs from LoadCommentInfo in that the information is fetched
	# from the web input form
	$CommentInfo{'article'}  = filterUserInput($cgi->param('article'));

	$CommentInfo{'email'} = filterUserInput($cgi->param('email')) || '';
	$CommentInfo{'author'} = filterUserInput($cgi->param('author')) || '';
	# eat pipe symbol (used to separate fields in $commentnum.inf)
	$CommentInfo{'email'} =~ s/\|//g;
	$CommentInfo{'email'} =~ s/^mailto\://;
	$CommentInfo{'author'} =~ s/\|//g;
	$CommentInfo{'origauthor'} = $CommentInfo{'author'};
	$CommentInfo{'site'} = $cgi->param('site') || '';
	GuessMailOrSite();

	$CommentInfo{'password'} = $cgi->param('password') || '';
	$CommentInfo{'text'} = $cgi->param('text') || '';
	$CommentInfo{'origtext'} = $CommentInfo{'text'};
	$CommentInfo{'headline'} = $cgi->param('headline') || '';
	$CommentInfo{'origheadline'} = $CommentInfo{'headline'};
}


sub SaveComment
{
	FetchCommentInfo();

	if ($CommentInfo{'article'} !~ /^\d+$/) { exit; }

	my $infoline = '';

	my $authornum = -1;
	my $usersfile = $basedir . 'users/users.db';
	if (-e $usersfile) {
		open USERS, "<$usersfile";
		while (<USERS>) {
			my ($usernum, $name) = split /\|/;
			chomp $name;
			if ($name eq $CommentInfo{'author'}) {
				$authornum = $usernum;
				last;
			}
		}
		close USERS;
	}

	my $time = localtime();
	FillDateTime($time);

	if ($authornum >= 0) {
		open PW, "<$basedir" . "users/$authornum.pwl";
		my $savedpw = <PW>;
		close PW;
		$savedpw = EncDec($savedpw);
		if ($savedpw ne $CommentInfo{'password'}) {
			OutputHTMLFile("usr/wrongpw.html");
			return;
		}
		open EMAIL, "<$basedir" . "users/$authornum.eml";
		my $registeredemail = <EMAIL>;
		close EMAIL;

		if ((!$CommentInfo{'email'}) || ($registeredemail eq $CommentInfo{'email'})) {
			$infoline = "REGISTERED$authornum\|$time\|seeeml";

			$CommentInfo{'email'} = $registeredemail;
		} else {
			$infoline = "REGISTERED$authornum\|$time\|$CommentInfo{'email'}";
		}
	} else {
		$infoline = "$CommentInfo{'author'}\|$time\|$CommentInfo{'email'}";
	}

	my $commentsfile = $basedir . 'comments/comments.db';

	my $lastcomment = -1;
	if (-e $commentsfile) {
		open COMMENTS, "<$commentsfile";
		while (<COMMENTS>) {
			(my $articlenum, $lastcomment) = split /\|/, $_;
		}
		close COMMENTS;
	}
	my $thiscomment = $lastcomment + 1;
	$CommentInfo{'commentnum'} = $thiscomment;

	open COMMENTS, ">>$commentsfile";
	print COMMENTS "$CommentInfo{'article'}\|$thiscomment\n";
	close COMMENTS;

	my $basepath = $basedir . "comments/$thiscomment";

	open INF,">>$basepath.inf";
	print INF "$infoline\n";
	close INF;

	open MSG, ">>$basepath.msg";
	print MSG "$CommentInfo{'text'}\n";
	close MSG;

	open HDL, ">>$basepath.hdl";
	print HDL "$CommentInfo{'headline'}\n";
	close HDL;

	open IP, ">>$basepath.ip";
	print IP "$ENV{'REMOTE_ADDR'}\n";
	close IP;

	if ($CommentInfo{'site'} ne '') {
		open SITE, ">>$basepath.sit";
		print SITE "$CommentInfo{'site'}\n";
		close SITE;
	}

	if ($showcommentsent) {
		OutputHTMLFile('commentsent.html');
	} else {
		ShowArticle($CommentInfo{'article'});
	}
}


sub OutputHTMLFile
{
	my $filename = $basedir . "html/" . shift;
	open HTMLFILE, "<$filename";
	while (<HTMLFILE>) {
		print SubstRefVars($_);
	}
	close HTMLFILE;
}


sub GetComments
{
	# return list of comment IDs for given article ID
	my $articlenum = pop;
	my $commentsdb = $basedir . 'comments/comments.db';
	my @thecomments = ();
	if (-e $commentsdb) {
		open COMMENTS, "<$commentsdb";
		my @comments = grep /^$articlenum\|/, <COMMENTS>;
		close COMMENTS;
		foreach my $comment (@comments) {
			my ($articlenum, $commentnum) = split /\|/, $comment;
			chomp $commentnum;
			if (!-e $basedir . "comments/$commentnum.del") {
				push @thecomments, $commentnum;
			}
		}
	}
	return @thecomments;
}

sub ArticleID
{
	# find article ID based on path (if necessary) and return ID
	my $article = pop;

	if ($article =~ /^\d+$/) {
		# ID already given
		return $article;
	}

	my $articlesdb = $basedir . 'comments/articles.db';
	my $articlefound = 0;
	my $articlenum = -1;
	my $articlepath = $articledir . $article;

	if (-e $articlepath) {
		if (-e $articlesdb) {
			open ARTICLES, "<$articlesdb";
			while (<ARTICLES>) {
				($articlenum, my $file) = split /\|/, $_;
				chomp $file;
				if ($article eq $file) {
					$articlefound = 1;
					last;
				}
			}
			close ARTICLES;
		}
		if (!$articlefound) {
			# create new ID if necessary
			open ARTICLES, ">>$articlesdb";
			$articlenum += 1;
			print ARTICLES "$articlenum\|$article\n";
			close ARTICLES;
		}
	}
	return $articlenum;
}

sub ArticlePath
{
	# convert article ID to path or return unchanged path (if given)
	my $article = pop;
	my $articlepath = '';
	my $articlesdb = $basedir . 'comments/articles.db';

	if ($article =~ /^\d+$/) {
		# ID was given - find path to respective article
		if (-e $articlesdb) {
			my $found = 0;
			open ARTICLES, "<$articlesdb";
			while (<ARTICLES>) {
				my ($num, $file) = split /\|/, $_;
				if ($num == $article) {
					chomp $file;
					$articlepath = $articledir . $file;
					$found = 1;
					last;
				}
			}
			close ARTICLES;
			if ($found == 0) {
				print "Invalid ID: $article .";
				exit;
			}
			return $articlepath;
		} else {
			print "Article database not created yet - please post comments before specifying IDs instead of alphanumeric filenames.";
			exit;
		}
	}

	# path was already given
	return $articledir . $article;
}

sub substituteCounts
{
	my $line = pop;

	while ($line =~ /\#\#COMMENTCOUNT_.*\#\#/) {
		(my $article = $line) =~ s/.*?\#\#COMMENTCOUNT_(.*?)\#\#.*/$1/s;
		my $count = scalar(GetComments(ArticleID($article)));
		if (!$count) { $count = 0; }
		my $countline = ($count == 1) ? $embed_commentcount : $embed_commentscount;
		$countline =~ s/\#\#number\#\#/$count/;
		$line =~ s/\#\#COMMENTCOUNT_.*?\#\#/$countline/;
	}

	if ($line =~ /\#\#EMBED_TOTALCOMMENTS\#\#/) {
		my $count = totalComments();
		(my $countline = $embed_totalcomments) =~ s/\#\#number\#\#/$count/;
		$line =~ s/\#\#EMBED_TOTALCOMMENTS\#\#/$countline/g;
	}


	return $line;
}

sub ShowMainNews
{
	my $main = shift;
	chomp $main;
	my $mainpath = $articledir . $main;

	if (!-e $mainpath) {
		OutputHTMLFile("notexistant.html");
		exit;
	}

	# substitute references to comment counts in main news
	print substituteCounts(FileToScalar($mainpath));
}

sub ShowArticle
{
	my $article = shift;
	chomp $article;

	my $articlepath = ArticlePath($article);
	$CommentInfo{'article'} = ArticleID($article);

	if (!-e $articlepath) {
		OutputHTMLFile("notexistant.html");
		exit;
	}

	(my $file = $articlepath) =~ s/.*\/(.*)/$1/;
	# Path Between ARTICLEDIR and file:
	(my $pbadaf = $articlepath) =~ s/$articledir(.*)$file/$1/;

	open ARTICLE, "<$articlepath";
	my $cssdone = 0;
	my $cssembed = FileToScalar($basedir . 'html/comments.css');
	my $comment_template =  FileToScalar($basedir . 'html/comment.tpl');

	while (<ARTICLE>) {
		# The following few lines are an attempt to change the URIs of
		# any embedded images (if not specified absolutely)
		$_ =~ s/src.*?\=.*?\"(.*?)\"/src\=\"\#\#$1\#\#\"/ig;
		while (/src\=\"##.*?\##"/i) {
			(my $imgname = $_) =~ s/.*?src\=\"\#\#(.*?)\#\#".*/$1/i;
			my $fullimgname = "";
			if ($imgname =~ /http\:\/\//) {
				$fullimgname = $imgname;
			} else {
				$fullimgname = $articledir . $pbadaf . $imgname;
			}
			chomp $fullimgname;
			$_ =~ s/src\=\"##.*?\##"/src\=\"$fullimgname\"/;
		}

		if (/\<style.*\>/i) {
			s/(.*\<style.*\>)(.*)/$1 . $cssembed . $2/i;
			$cssdone = 1;
		}
		if (/\<\/head\>/i) {
			if (!$cssdone) {
				my $styletag = "\<style type\=\"text\/css\"\>";
				s/(.*)(\<\/head\>)(.*)/$1$styletag\n$cssembed\n\<\/style\>\n$2$3/i;
			}
		}

		$_ = substituteCounts($_);

		if (/\<\/body\>/i) {
			# time to attach the available comments
			(my $beforebody = $_) =~ s/(.*)\<\/body\>.*/$1/i;
			print $beforebody;
			(my $afterbody = $_) =~ s/.*\<\/body\>(.*)/$1/i;
			print FileToScalar($basedir . 'html/comments.sep');
			my @comments = GetComments($CommentInfo{'article'});
			foreach my $commentid (@comments) {
				LoadCommentInfo($commentid);
				print SubstRefVars($comment_template);
			}
			my $form = FileToScalar($basedir .'html/newcomment.tpl');
			print SubstRefVars($form);
			print "</body>$afterbody";
			next;
		}
		print;
	}
	close ARTICLE;
}

sub AddUser
{
	my $newuser = filterUserInput($cgi->param('name'));
	# eat pipe symbol (used to separate fields)
	$newuser =~ s/\|//g;
	my $newemail = $cgi->param('email');
	my $newpw = $cgi->param('newpw');
	my $retypedpw = $cgi->param('retpw');

	my $lastipfile = $basedir . 'users/lastip';
	if (-e $lastipfile) {
		open LASTIP, "<$lastipfile";
		my $lastip = <LASTIP>;
		chomp $lastip;
		close LASTIP;
		# This could be used to disallow registering too many names
		# if ($lastip eq $ENV{'REMOTE_ADDR'}) {
		#	OutputHTMLFile("none", 'usr/onepip.html');
		#	return;
		# }
	}


	chomp $newuser;
	my $authornum = -1;
	my $lastusernum = -1;

	my $usersdb = $basedir . 'users/users.db';
	if (-e $usersdb) {
		open USERS, "<$usersdb";
		while (<USERS>) {
			my ($usernum, $name) = split /\|/;
			chomp $name;
			if ($name eq $newuser) {
				$authornum = $usernum;
			}
			$lastusernum = $usernum;
		}
		close USERS;
	}
	if ($authornum >= 0) {
		OutputHTMLFile('usr/notfree.html');
		return;
	}

	if ($newpw ne $retypedpw) {
		OutputHTMLFile('usr/nopwmatch.html');
		return;
	}

	my $newusernum = $lastusernum + 1;
	open USERS, ">>$basedir" . "users/users.db";
	print USERS "$newusernum\|$newuser\n";
	close USERS;

	open EMAIL, ">>$basedir" . "users/$newusernum.eml";
	print EMAIL "$newemail";
	close EMAIL;

	my $encpw = EncDec($newpw);
	open PW, ">$basedir" . "users/$newusernum.pwl";
	print PW "$encpw";
	close PW;

	# For better efficiency, write user name also to separate file
	open SYNTAX, ">$basedir" . "users/$newusernum.syn";
	print SYNTAX "$newuser";
	close SYNTAX;

	open LASTIP, ">$basedir" . "users/lastip";
	print LASTIP "$ENV{'REMOTE_ADDR'}";
	close LASTIP;

	OutputHTMLFile("usr/registered.html");
}


sub totalComments
{
	my $commentsfile = $basedir . 'comments/comments.db';
	if (!-e $commentsfile) { return 0; }
	open FH, "<$commentsfile";
	my @comments = <FH>;
	close FH;
	my $count = 0;
	foreach my $comment (@comments) {
		my ($articlenum, $commentnum) = split /\|/, $comment;
		chomp $commentnum;
		my $delpath = $basedir . "comments/$commentnum.del";
		if (-e $delpath) {
			next;
		}
		$count++;
	}
	return $count;
}



# Execution-entrypoint


# Read config-file

if (-e "cgicomment.conf") {
	open CONFIG, "<cgicomment.conf";
	while (<CONFIG>) {
		if (/^\#/ || /^\s+$/) { next; }
		chomp;
		(my $variable, my $value) = split (/=/, $_, 2);
		if ($variable =~ /^basedir/i) {
			$value =~ s/(.*)\s+$/$1/;
			if ( substr($value, -1, 1) ne '/') { $value .= '/'; }
			$basedir = $value;
		} elsif ($variable =~ /^articledir/i) {
			$value =~ s/(.*)\s+$/$1/;
			if ( substr($value, -1, 1) ne '/') { $value .= '/'; }
			$articledir = $value;
		} elsif ($variable =~ /^weekdays/i) {
			my @weekdays = split /\s+/, $value;
			my $counter = 0;
			foreach my $wd (@weekdays) {
				$FullWeekDay{$ShortWeekDays[$counter]} = $wd;
				$counter++;
			}
		} elsif ($variable =~ /^showcommentsent/i) {
			$value =~ s/(.*)\s+$/$1/;
			$showcommentsent = $value;
		} elsif ($variable =~ /^commentcount/i) {
			$commentcount = $value;
		} elsif ($variable =~ /^commentscount/i) {
			$commentscount = $value;
		} elsif ($variable =~ /^embed\_commentcount/i) {
			$embed_commentcount = $value;
		} elsif ($variable =~ /^embed\_commentscount/i) {
			$embed_commentscount = $value;
		} elsif ($variable =~ /^totalcomments/i) {
			$totalcomments = $value;
		} elsif ($variable =~ /^embed_totalcomments/i) {
			$embed_totalcomments = $value;
		}
	}
	close CONFIG;
}


print "Content-type: text/html\n\n";

$CommentInfo{'basedir'} = $basedir;

my $action = $cgi->param('action');

if (!$action || ($action eq 'showarticle')) {
	my $article = filterUserInput($cgi->param('article'));
	if (defined $article) {
		ShowArticle($article);
	} else {
		print "No article specified. Use cgicomment.pl?article=anyfile.html\n";
	}
} elsif ($action eq 'addcomment') {
	my $preview = $cgi->param('action1') || '__nopreview';
	if ($preview eq '__nopreview') {
		SaveComment();
	} else {
		# invoked via preview-button - don't save, but show preview
		FetchCommentInfo();
		FillDateTime();
		OutputHTMLFile('preview.html');
	}
} elsif ($action eq 'adduser') {
	AddUser();
} elsif ($action eq 'showmain') {
	my $mainnews = filterUserInput($cgi->param('mainnews'));
	if (defined $mainnews) {
		ShowMainNews($mainnews);
	} else {
		print "No main news file specified. Use cgicomment.pl?action=showmain&mainnews=your_mainnews.html\n";
	}
} elsif ($action eq 'commentcount') {
	my $article = filterUserInput($cgi->param('article'));
	if (defined $article) {
		my $count = scalar(GetComments(ArticleID($article)));
		if (!$count) { $count = 0; }
		my $countline = ($count == 1) ? $commentcount : $commentscount;
		$countline =~ s/\#\#number\#\#/$count/;
		print $countline;
	} else {
		print "No article specified. Use cgicomment.pl?article=anyfile.html&action=commentcount\n";
	}
} elsif ($action eq 'totalcomments') {
	my $count = totalComments();
	(my $countline = $totalcomments) =~ s/\#\#number\#\#/$count/;
	print $countline;
}
